<?php

// Exit if accessed directly.
if (! defined('ABSPATH')) {
	exit;
}

class WC_Gateway_Nomod extends WC_Payment_Gateway
{
	public $enabled;
	public $secret_key;
	public $webhook_secret;
	public $id = 'nomod';
	const MAX_TITLE_LENGTH = 100;

	public function __construct()
	{
		$this->has_fields = false;
		$this->supports   = array(
			'products',
		);
		$this->method_title       = __('Nomod', 'nomod-for-woocommerce');
		$this->method_description = __('Nomod functions as a payment option during checkout for customers in your store, allowing them to pay seamlessly using Visa, Mastercard, American Express, Apple Pay, Google Pay, Tabby, & Tamara.', 'nomod-for-woocommerce');

		$this->title       		  = $this->get_option('title', __('Visa, Mastercard, American Express, Apple Pay, Google Pay, Tabby, & Tamara', 'nomod-for-woocommerce'));
		$this->secret_key         = $this->get_validated_option('secret_key');
		WC_Nomod_API::set_secret_key($this->secret_key);

		$this->init_form_fields();
		$this->init_settings();

		// add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
		add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'validate_and_process_admin_options'));
	}

	public function init_form_fields()
	{
		$this->form_fields = array(
			'enabled' => array(
				'title'   => __('General', 'nomod-for-woocommerce'),
				'description' => 'When enabled, payment methods powered by Nomod will appear on checkout.',
				'type'    => 'checkbox',
				'label'   => __('Enable Nomod', 'nomod-for-woocommerce'),
				'default' => 'yes',
			),
			'title' => array(
				'title'       => __('Title', 'nomod-for-woocommerce'),
				'type'        => 'text',
				'description' => __('This controls the title shown to customers during checkout.', 'nomod-for-woocommerce'),
				'default'     => __('Visa, Mastercard, American Express, Apple Pay, Google Pay, Tabby, & Tamara', 'nomod-for-woocommerce'),
			),
			'secret_key' => array(
				'title'       => 'API Key',
				'type'        => 'password'
			)
		);
	}

	public function validate_and_process_admin_options()
	{
		$api_key = filter_input(INPUT_POST, 'woocommerce_nomod_secret_key', FILTER_SANITIZE_SPECIAL_CHARS);
		$title = filter_input(INPUT_POST, 'woocommerce_nomod_title', FILTER_SANITIZE_SPECIAL_CHARS);

		if (empty($title)) {
			WC_Admin_Settings::add_error(WC_Nomod_Messages::get_message('empty_title'));
			return;
		}

		$title = sanitize_text_field($title);

		if (strlen($title) > self::MAX_TITLE_LENGTH) {
			WC_Admin_Settings::add_error(WC_Nomod_Messages::get_message('invalid_title'));
			return;
		}

		// If API key is not changed we just process the admin options
		if ($this->secret_key === $api_key) {
			$this->title = $title;
			self::update_nomod_settings('title', $title);
			$this->process_admin_options();
			return;
		}

		if (empty($api_key)) {
			WC_Admin_Settings::add_error(WC_Nomod_Messages::get_message('empty_api_key'));
			return;
		}

		try {
			// Check if the API key is different from before
			if ($this->secret_key !== $api_key) {
				self::update_nomod_settings('secret_key', $api_key);
				WC_Nomod_API::set_secret_key($api_key);
				$response = WC_Nomod_Payment_Webhook::create_webhook();

				if (isset($response['key'])) {
					self::update_nomod_settings('webhook_secret', $response['key']);
				}
			}

			$this->title = $title;
			self::update_nomod_settings('title', $title);
			$this->process_admin_options();
		} catch (\Exception $e) {
			$logger = wc_get_logger();
			$logger->error('Error when saving API key: ', array('message' => $e->getMessage()));
			WC_Admin_Settings::add_error(!empty($e->getMessage()) ? $e->getMessage() : WC_Nomod_Messages::get_message('invalid_api_key'));

			// Reset the secret key to the old value
			self::update_nomod_settings('secret_key', $this->secret_key);
			WC_Nomod_API::set_secret_key($this->secret_key);
			return;
		}
	}

	public function process_payment($order_id)
	{
		WC_Nomod_Business::get_business();

		$customer = WC_Nomod_Customer::create_customer_if_not_exists($order_id);
		$customer_id = isset($customer['id']) ? $customer['id'] : null;

		if (!$customer_id) {
			throw new Exception(esc_html(WC_Nomod_Messages::get_message('no_customer')));
		}

		$link = WC_Nomod_Checkout::generate_checkout_link($order_id, $customer_id);
		$redirect_url = isset($link['url']) ? $link['url'] : null;

		if (null === $redirect_url) {
			throw new Exception(esc_html(WC_Nomod_Messages::get_message('no_payment_link')));
		}

		return array(
			'result'   => 'success',
			'redirect' => $redirect_url
		);
	}

	public function get_validated_option($field_key, $empty_value = null)
	{
		$value = parent::get_option($field_key, $empty_value);
		return $this->validate_field($value);
	}

	private function validate_field($field_value)
	{
		if (empty($field_value)) {
			return '';
		}

		$value = sanitize_text_field($field_value);
		return $value;
	}

	public function update_nomod_settings($key, $value)
	{
		$options = get_option('woocommerce_nomod_settings');

		// Check if options exist and unserialize them
		if ($options) {
			$settings = maybe_unserialize($options);
		} else {
			// If no options exist, initialize an empty array
			$settings = array();
		}

		$settings[$key] = $value;

		$update_response = update_option('woocommerce_nomod_settings', $settings);

		return $update_response;
	}
}
