<?php
if (! defined('ABSPATH')) {
    exit;
}

class WC_Nomod_API
{

    const TIMEOUT = 70;
    const BASE_URL = 'https://api.nomod.com/';
    const API_VERSION = 'v1';

    private static $secret_key = '';

    public static function set_secret_key($secret_key)
    {
        self::$secret_key = $secret_key;
    }

    public static function get_secret_key()
    {
        if (! self::$secret_key) {
            $secret_key      = get_option('secret_key') ?? '';
            self::set_secret_key($secret_key);
        }
        return self::$secret_key;
    }

    public static function get_headers()
    {
        $headers = apply_filters(
            'woocommerce_nomod_request_headers',
            [
                'X-API-KEY'    => self::get_secret_key(),
                'Content-Type' => 'application/json'
            ]
        );
        return $headers;
    }

    public static function post($body, $api = '', $method = 'POST')
    {
        $logger  = wc_get_logger();
        $headers = self::get_headers();
        $body    = wp_json_encode(apply_filters('woocommerce_nomod_request_body', $body, $api));

        $response = wp_safe_remote_post(
            self::BASE_URL . self::API_VERSION . '/' . $api,
            [
                'method'  => $method,
                'headers' => $headers,
                'body'    => $body,
                'timeout' => self::TIMEOUT,
            ]
        );

        if (is_wp_error($response)) {
            $error_message = $response->get_error_message();
            $logger->error('Error sending request to Nomod API: ', array('endpoint' => $api, 'details' => $error_message));
            throw new Exception(esc_html(WC_Nomod_Messages::get_message('network_error')));
        }

        return self::handle_api_response($response, $api);
    }

    public static function get($api)
    {
        $logger = wc_get_logger();
        $headers = self::get_headers();
        $response = wp_safe_remote_get(
            self::BASE_URL . self::API_VERSION . '/' . $api,
            [
                'method'  => 'GET',
                'headers' => $headers,
                'timeout' => self::TIMEOUT,
            ]
        );

        if (is_wp_error($response)) {
            $error_message = $response->get_error_message();
            $logger->error('Error sending request to Nomod API: ', array('endpoint' => $api, 'details' => $error_message));
            throw new Exception(esc_html(WC_Nomod_Messages::get_message('network_error')));
        }

        return self::handle_api_response($response, $api);
    }

    private static function handle_api_response($response, $endpoint)
    {
        $logger = wc_get_logger();
        $status_code = wp_remote_retrieve_response_code($response);
        $response_body = wp_remote_retrieve_body($response);
        $decoded_response_body = json_decode($response_body, true);

        if ($status_code >= 200 && $status_code < 300) {
            return $decoded_response_body;
        } else if ($status_code >= 400 && $status_code < 500) {
            $logger->error('Client error: ', array(
                'code' => $status_code,
                'endpoint' => $endpoint,
                'details' => $decoded_response_body,
            ));

            throw new Exception(esc_html(self::getFirstErrorMessage($decoded_response_body)));
        } else {
            $logger->error('Internal server error: ', array(
                'code' => $status_code,
                'endpoint' => $endpoint,
                'details' => $decoded_response_body,
            ));
            throw new Exception(esc_html(self::getFirstErrorMessage($decoded_response_body)));
        }
    }

    private static function getFirstErrorMessage($decoded_response_body)
    {
        if (is_array($decoded_response_body) && !empty($decoded_response_body)) {
            $first_field = reset($decoded_response_body);

            if (isset($first_field['message'])) {
                return  $first_field['message'];
            }
        }

        return WC_Nomod_Messages::get_message('something_wrong');
    }
}
