<?php
if (! defined('ABSPATH')) {
    exit;
}

class WC_Nomod_Payment_Webhook
{
    const WEBHOOK_URL = '/wc-api/wc_nomod_webhook';

    public static function create_webhook()
    {
        $base_url = get_site_url();
        $payload = [
            'webhook_url' => $base_url . self::WEBHOOK_URL
        ];

        return WC_Nomod_API::post($payload, 'woocommerce/create-webhook');
    }

    public static function handle_nomod_payment_webhook()
    {
        if (!isset($_SERVER['CONTENT_TYPE']) || $_SERVER['CONTENT_TYPE'] !== 'application/json') {
            self::send_webhook_response('Invalid content type for the data');
            exit;
        }

        $is_valid =  self::validate_webhook_request();

        if (!$is_valid) exit;

        $data = json_decode(file_get_contents('php://input'), true);

        if (json_last_error() !== JSON_ERROR_NONE || empty($data)) {
            self::send_webhook_response('Invalid JSON data provided');
            exit;
        }

        $order_id = isset($data['order_id']) ? sanitize_text_field($data['order_id']) : '';
        $idempotency_key = isset($data['idempotency_key']) ? sanitize_text_field($data['idempotency_key']) : '';
        $payment_status = isset($data['payment_status']) ? sanitize_text_field($data['payment_status']) : '';
        $valid_statuses = WC_Nomod_Checkout::get_invoice_statuses();

        if (!in_array($payment_status, $valid_statuses, true)) {
            self::send_webhook_response('Invalid payment status received');
            exit;
        }

        $is_body_valid =  self::validate_webhook_body($idempotency_key, $payment_status, $order_id);
        if (!$is_body_valid) exit;

        $order = wc_get_order($order_id);

        if (!$order) {
            self::send_webhook_response('Order not found with id: ' . $order_id, 404);
            exit;
        }

        $processed_idempotency_keys = $order->get_meta('_nomod_idempotency_keys', true) ?: [];

        if (in_array($idempotency_key, $processed_idempotency_keys, true)) {
            self::send_webhook_response('Request already processed. Ignoring duplicate request', 200, true);
            exit;
        }

        if ($payment_status === 'paid') {
            $order->payment_complete();
            WC()->cart->empty_cart();

            // Store the idempotency key
            $sanitized_idempotency_key = sanitize_key($idempotency_key);
            $processed_idempotency_keys[] = $sanitized_idempotency_key;
            $order->update_meta_data('_nomod_idempotency_keys', $processed_idempotency_keys);
            $order->save();

            self::send_webhook_response('Order completed successfully', 200, true);
            exit;
        } else {
            $order->update_status('failed', __('Payment failed via webhook.', 'nomod-for-woocommerce'));

            // Store the idempotency key
            $sanitized_idempotency_key = sanitize_key($idempotency_key);
            $processed_idempotency_keys[] = $sanitized_idempotency_key;
            $order->update_meta_data('_nomod_idempotency_keys', $processed_idempotency_keys);
            $order->save();

            self::send_webhook_response('Payment failed for order_id: ' . $order_id, 200);
            exit;
        }
    }

    private static function validate_webhook_request()
    {
        // Validate the request
        $svix_id = isset($_SERVER['HTTP_SVIX_ID']) ? sanitize_text_field(wp_unslash($_SERVER['HTTP_SVIX_ID'])) : null;
        $svix_timestamp = isset($_SERVER['HTTP_SVIX_TIMESTAMP']) ? sanitize_text_field(wp_unslash($_SERVER['HTTP_SVIX_TIMESTAMP'])) : null;
        $body = file_get_contents('php://input');
        $settings = get_option('woocommerce_nomod_settings');
        $secret = isset($settings['webhook_secret']) ? $settings['webhook_secret'] : '';

        if (empty($svix_id) || empty($svix_timestamp)) {
            self::send_webhook_response('Missing required headers');
            return false;
        }

        // Construct the signed content
        $signed_content = $svix_id . "." . $svix_timestamp . "." . $body;

        // Extract and decode the secret key
        $secret_key = base64_decode(explode('_', $secret)[1]);

        // Generate the HMAC SHA256 signature and encode it in base64
        $calculated_signature = base64_encode(hash_hmac('sha256', $signed_content, $secret_key, true));

        // Get the `svix-signature` header from the request to compare with the calculated signature
        $svix_signature = isset($_SERVER['HTTP_SVIX_SIGNATURE']) ? sanitize_text_field(wp_unslash($_SERVER['HTTP_SVIX_SIGNATURE'])) : null;

        if ($svix_signature) {
            // Extract the part after "v1," for comparison
            $svix_signature = explode(',', $svix_signature)[1] ?? null;
        }

        // Compare the calculated signature with the provided signature
        if (!hash_equals($calculated_signature, $svix_signature)) {
            self::send_webhook_response('Invalid signature', 403);
            return false;
        }

        return true;
    }

    private static function validate_webhook_body($idempotency_key, $payment_status, $order_id)
    {
        $missing_fields = [];

        if (empty($order_id) || !ctype_digit($order_id)) {
            $missing_fields[] = 'order_id';
        }

        if (empty($payment_status)) {
            $missing_fields[] = 'payment_status';
        }

        if (empty($idempotency_key)) {
            $missing_fields[] = 'idempotency_key';
        }

        if (!empty($missing_fields)) {
            self::send_webhook_response('Invalid data provided. Missing required fields: ' . implode(', ', $missing_fields) . '.');
            return false;
        }

        return true;
    }

    private static function send_webhook_response($message, $status_code = 400, $success = false)
    {
        $response =  [
            'success' => $success,
            'message' => $message,
        ];

        wp_send_json($response, $status_code);
    }
}
