<?php

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit;
}

class WC_Nomod_Utils
{
    public static function truncate_string($string, $length = 100)
    {
        if (strlen($string) > $length) {
            return substr($string, 0, $length);
        }
        return $string;
    }

    public static function find_currency_by_code($currencies, $currency_code)
    {
        foreach ($currencies as $currency) {
            if ($currency['code'] === $currency_code) {
                return $currency;
            }
        }

        return null;
    }

    /**
     * Prepare order items for processing.
     *
     * @param WC_Order $order The WooCommerce order object.
     *
     * @return array{0: array<int, array{name: string, amount: string, quantity: int}>, 
     *               1: array<int, array{name: string, mode: string, value: string, mode_value: float}>, 
     *               2: float} 
     *         Returns an array containing:
     *         - Items: List of products and fees as associative arrays.
     *         - Taxes: List of tax details.
     *         - Discounts: Discount amounts applied.
     */
    public static function prepare_order_items($order)
    {
        $order_items = $order->get_items();
        $order_fees = $order->get_items('fee');
        $order_taxes = $order->get_items('tax');

        if (empty($order_items)) {
            return [];
        }

        $shipping_fee = self::get_nomod_amount($order->get_shipping_total(), false);
        $order_discount = self::get_nomod_amount($order->get_discount_total()) ?? 0;

        $items = [];
        $taxes = [];
        $discounts = $order_discount;

        // Add line items
        foreach ($order_items as $item) {
            if ($item instanceof WC_Order_Item_Product) {
                $product_name = $item->get_name();
                $product = $item->get_product();
                $product_amount = self::get_nomod_amount($product->get_price(), false);
                $product_quantity = $item->get_quantity();

                $items[] = [
                    'name' => WC_Nomod_Utils::truncate_string($product_name),
                    'amount' => $product_amount,
                    'quantity' => $product_quantity
                ];
            }
        }

        // Add fees
        foreach ($order_fees as $fee) {
            if ($fee instanceof WC_Order_Item_Fee) {
                $fee_total = self::get_nomod_amount($fee->get_total(), false);

                // Add positive fee as line item 
                if ((float) $fee_total > 0) {
                    $fee_name = $fee->get_name();

                    $items[] = [
                        'name' => WC_Nomod_Utils::truncate_string($fee_name),
                        'amount' => $fee_total,
                        'quantity' => 1
                    ];
                } else {
                    // Add negative fee as discount
                    $item_discount = self::get_nomod_amount($fee->get_total());

                    if ($item_discount < 0) {
                        $discounts += abs($item_discount);
                    }
                }
            }
        }

        // Add taxes
        foreach ($order_taxes as $tax) {
            if ($tax instanceof WC_Order_Item_Tax) {
                $tax_label = $tax->get_label();
                $tax_total = self::get_nomod_amount($tax->get_tax_total());
                $tax_rate  = self::get_nomod_amount($tax->get_rate_percent());
                $tax_ship_total = self::get_nomod_amount($tax->get_shipping_tax_total());

                if ($tax_total > 0) {
                    $taxes[] = [
                        'name'          => WC_Nomod_Utils::truncate_string($tax_label),
                        'mode'          => 'percentage',
                        'value'         => self::get_nomod_amount(($tax_total + $tax_ship_total), false),
                        'mode_value'    => $tax_rate,
                    ];
                }
            }
        }

        // Add shipping fee if applicable
        if ((float) $shipping_fee > 0) {
            $items[] = [
                'name' => __('Shipping Fee', 'nomod-for-woocommerce'),
                'amount' => $shipping_fee,
                'quantity' => 1
            ];
        }

        return [$items, $taxes, $discounts];
    }

    public static function get_success_url($order)
    {
        $order_id = $order->get_id();
        $order_key = $order->get_order_key();

        $success_url = wc_get_endpoint_url(
            'order-received',
            $order_id,
            wc_get_checkout_url()
        );

        return add_query_arg('key', $order_key, $success_url);
    }

    public static function get_failure_url()
    {
        $checkout_url = wc_get_checkout_url();
        return add_query_arg('payment_status', 'failed', $checkout_url);
    }

    public static function get_nomod_amount($raw_amount, $as_float = true)
    {
        if ($as_float) return (float) wc_format_decimal((float) $raw_amount, 2, false);
        return wc_format_decimal((float) $raw_amount, 2, false);
    }
}
